﻿using System.Collections.Generic;
using System.ComponentModel;
using DevExpress.ExpressApp.DC;
using DevExpress.Persistent.Base;
using DevExpress.Xpo;
using Xafari.Base;
using Xafari.DC;

namespace NonPersistentCollections.Module.BusinessObjects
{
	// Пример показывает как описывать связь 1:N для
	//	- Master - NonPersistent
	//  - Details - Persistent

	[DomainComponent]
	[NonPersistentDc]
	public interface Master1
	{
		string Master1String1 { get; set; }

		[NonPersistentDc]
		[DevExpress.ExpressApp.DC.Aggregated]
		[BackReferenceProperty("Master1")]
		IList<Details1> Details1 { get; }

		[NonPersistentDc]
		[DevExpress.ExpressApp.DC.Aggregated]
		[BackReferenceProperty("Master1")]
		IList<Details2> Details2 { get; }

		[NonPersistentDc]
		[DevExpress.ExpressApp.DC.Aggregated]
		[BackReferenceProperty("Master1")]
		IList<Details3> Details3 { get; }
	}

	[DomainLogic(typeof(Master1))]
	public class Master1Logic : DomainLogicBase<Master1>
	{
		public Master1Logic(Master1 instance)
			: base(instance)
		{
		}

		public IList<Details1> Details1
		{
			get { return this.GetWeakList<Details1>("Details1"); }
		}

		public IList<Details2> Details2
		{
			get { return this.GetWeakList<Details2>("Details2"); }
		}

		public IList<Details3> Details3
		{
			get { return this.GetWeakList<Details3>("Details3"); }
		}
	}

	[NonPersistentDc]
	[DomainComponent]
	public interface DetailsBase : OwnerSupport
	{
		[Browsable(false)]
		[PersistentDc]
		[Term("Owner")]
		XPWeakReferenceStruct Master1 { get; set; }
	}

	[DomainComponent]
	public interface Details1 : OwnerSupport
	{
		string Details1String1 { get; set; }

		[Browsable(false)]
		[DevExpress.ExpressApp.DC.Aggregated]
		[Term("Owner")]
		XPWeakReference Master1 { get; set; }
	}

	[DomainComponent]
	public interface Details2 : DetailsBase
	{
		string Details2String1 { get; set; }
	}

	[DomainComponent]
	public interface Details3 : DetailsBase
	{
		string Details3String1 { get; set; }
	}

	[DefaultClassOptions]
	[DomainComponent]
	public interface DerivedMaster1 : Master1
	{
		
	}

	[DefaultClassOptions]
	[DomainComponent]
	public interface OneMoreDerivedMaster1 : Master1
	{

	}
}