﻿using System.Collections.Generic;
using System.ComponentModel;
using DevExpress.ExpressApp.DC;
using DevExpress.Persistent.Base;
using Xafari.Base;
using Xafari.DC;

namespace NonPersistentCollections.Module.BusinessObjects
{
	// Пример показывает как описывать связь 1:N для
	//	- Master - NonPersistent
	//  - Details - NonPersistent

	[DomainComponent]
	[NonPersistentDc]
	public interface Master2
	{
		string Master2String1 { get; set; }

		[NonPersistentDc]
		[Browsable(false)]
		IList<Master2Details1> Details1 { get; }

		[NonPersistentDc]
		[Browsable(false)]
		IList<Master2Details2> Details2 { get; }
	}

	[DomainLogic(typeof(Master2))]
	public class Master2Logic : DomainLogicBase<Master2>
	{
		public Master2Logic(Master2 instance) : base(instance)
		{
		}

		public IList<Master2Details1> Details1
		{
			get { return this.Instance.GetListValueByTerm<Master2Details1>("Details1"); }
		}

		public IList<Master2Details2> Details2
		{
			get { return this.Instance.GetListValueByTerm<Master2Details2>("Details2"); }
		}
	}

	[DomainComponent]
	[DefaultClassOptions]
	public interface DerivedMaster2 : Master2
	{
		[BackReferenceProperty("DerivedMaster2")]
		[Aggregated]
		[Term("Details1")]
		IList<DerivedMaster2Details1> DerivedDetails1 { get; }

		[BackReferenceProperty("_Master2")]
		[Aggregated]
		[Term("Details2")]
		IList<_Master2Details2> _Details2 { get; }
	}

	[DomainComponent]
	[DefaultClassOptions]
	public interface OneMoreDerivedMaster2 : Master2
	{
		[BackReferenceProperty("OneMoreDerivedMaster2")]
		[Aggregated]
		[Term("Details1")]
		IList<OneMoreDerivedMaster2Details1> OneMoreDerivedDetails1 { get; }
	}

	[DomainComponent]
	[NonPersistentDc]
	public interface Master2Details1
	{
		string Details1String1 { get; set; }

		[Browsable(false)]
		Master2 Master2 { get; set; }
	}

	[DomainLogic(typeof(Master2Details1))]
	public class Master2Details1Logic : DomainLogicBase<Master2Details1>
	{
		public Master2Details1Logic(Master2Details1 instance)
			: base(instance)
		{
		}

		public Master2 Master2
		{
			get { return this.Instance.GetValueByTerm<Master2>("Master2"); }
			set { this.Instance.SetValueByTerm("Master2", value);}
		}
	}

	[DomainComponent]
	public interface DerivedMaster2Details1 : Master2Details1
	{
		[Term("Master2")]
		DerivedMaster2 DerivedMaster2 { get; set; }
	}

	[DomainComponent]
	public interface OneMoreDerivedMaster2Details1 : Master2Details1
	{
		[Term("Master2")]
		OneMoreDerivedMaster2 OneMoreDerivedMaster2 { get; set; }
	}

	[DomainComponent]
	[NonPersistentDc]
	public interface Master2Details2
	{
		string Details2String1 { get; set; }

		[Browsable(false)]
		Master2 Master2 { get; set; }
	}

	[DomainLogic(typeof(Master2Details2))]
	public class Master2Details2Logic : DomainLogicBase<Master2Details2>
	{
		public Master2Details2Logic(Master2Details2 instance)
			: base(instance)
		{
		}

		public Master2 Master2
		{
			get { return this.Instance.GetValueByTerm<Master2>("Master2"); }
			set { this.Instance.SetValueByTerm("Master2", value); }
		}
	}

	[DomainComponent]
	public interface _Master2Details2 : Master2Details2
	{
		[Term("Master2")]
		DerivedMaster2 _Master2 { get; set; }
	}
}