﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using DevExpress.Data.Filtering;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.DC;
using DevExpress.ExpressApp.Xpo;
using DevExpress.Xpo;
using DevExpress.Xpo.Helpers;
using NonPersistentCollections.Module;

namespace Xafari.DC
{
	[Obsolete("Use DomainLogicBase<T>", true)]
	public class _DomainLogicBase<T> : DomainLogicBase<T>
		where T : class
	{
		public _DomainLogicBase(T instance)
			: base(instance)
		{

		}

	}

	/// <summary>
	/// Базовый класс для реализации логики доменных компонентов.
	/// </summary>
	/// <typeparam name="T">Тип доменного компонента, для которого создается логика.</typeparam>
	public class DomainLogicBase<T> : IDisposable
		where T : class 
	{
		private ITypeInfo _typeInfo;
		private IObjectSpace _objectSpace;
		private Dictionary<IMemberInfo, object> _customPropertyStore;

		public DomainLogicBase(T instance)
		{
		    if (instance == null) throw new ArgumentNullException("instance");

		    this.Instance = instance;
		}

		/// <summary>
		/// Возвращает ссылку на объект.
		/// </summary>
		protected T Instance { get; private set; }

		/// <summary>
		/// Возвращает <see cref="IObjectSpace"/>, в котором создан объект.
		/// </summary>
		protected IObjectSpace ObjectSpace
		{
		    get
		    {
		        return _objectSpace ?? (_objectSpace = XPObjectSpace.FindObjectSpaceByObject(this.Instance));
		    }
		}

		/// <summary>
		/// Возвращает <see cref="ITypeInfo"/> для объекта.
		/// </summary>
		protected ITypeInfo TypeInfo
		{
		    get
		    {
		        return this._typeInfo ?? (this._typeInfo = XafTypesInfo.Instance.FindTypeInfo(this.Instance.GetType()));
		    }
		}

		public virtual void Dispose()
		{
			this.Instance = null;
			this._objectSpace = null;

			if (this._customPropertyStore != null)
			{
				foreach (var list in this._customPropertyStore.Values.OfType<IDisposable>())
					list.Dispose();

				this._customPropertyStore.Clear();
				this._customPropertyStore = null;
			}
		}

		/// <summary>
		/// Возвращает Id для объекта.
		/// </summary>
	    protected object InstanceId
	    {
			get
			{
#warning получаю из сессии пока не устранена проблема http://www.devexpress.com/Support/Center/Question/Details/B231282
				return (this.Instance as ISessionProvider).Session.GetKeyValue(this.Instance);
				//this.ObjectSpace.GetKeyValue(this.Instance); 
			}
	    }

		protected Dictionary<IMemberInfo, object> CustomPropertyStore
		{
		    get
		    {
		        return this._customPropertyStore ?? (this._customPropertyStore = new Dictionary<IMemberInfo, object>());
		    }
		}

		protected IList<TDetails> GetWeakList<TDetails>(string memberName)
		{
			var memberInfo = this.TypeInfo.FindMember(memberName);
			if (memberInfo == null)
				throw new InvalidOperationException(string.Format("Can't find member '{0}' for '{1}'", memberName, this.TypeInfo));

			object obj2;
			this.CustomPropertyStore.TryGetValue(memberInfo, out obj2);
            
            var list = (XPCollection<TDetails>)obj2;
			if (list == null)
			{
				list = this.CreateWeakList<TDetails>(memberInfo);
				this.CustomPropertyStore[memberInfo] = list;
			}

			return list;
		}

		protected virtual XPCollection<TDetails> CreateWeakList<TDetails>(IMemberInfo memberInfo)
		{
			var backRefernceAttribute = memberInfo.FindAttribute<BackReferencePropertyAttribute>();
			if (backRefernceAttribute == null)
				throw new InvalidOperationException(string.Format("Can't find '{0}' for member '{1}'",
				                                                  typeof (BackReferencePropertyAttribute).FullName, memberInfo));
			var backReferenceMemberInfo = memberInfo.ListElementTypeInfo.FindMember(backRefernceAttribute.PropertyName);
			if (backReferenceMemberInfo == null)
				throw new InvalidOperationException(
					string.Format("Can't find member '{0}' for '{1}' specified by '{2}' for member '{3}'",
					              backRefernceAttribute.PropertyName, memberInfo.ListElementTypeInfo,
					              typeof (BackReferencePropertyAttribute).FullName, memberInfo));

			var session = ((XPObjectSpace) this.ObjectSpace).Session;
			var criteria =
				CriteriaOperator.Parse(string.Format("{0}.TargetType = ? AND {0}.TargetKey = ?", backReferenceMemberInfo.Name),
				                       session.GetObjectType(this.Instance),
				                       XPWeakReference.KeyToString(this.ObjectSpace.GetKeyValue(this.Instance)));
			var list = new XPCollection<TDetails>(session, criteria);

			if (backReferenceMemberInfo.MemberType == typeof (XPWeakReference))
				list.ListChanged += (sender, args) =>
					{
						if (args.ListChangedType != ListChangedType.ItemAdded) return;

						var obj = list[args.NewIndex];
						if (backReferenceMemberInfo.GetValue(obj) == null)
							backReferenceMemberInfo.SetValue(obj, this.ObjectSpace.CreateObject<XPWeakReference>());
						((XPWeakReference) backReferenceMemberInfo.GetValue(obj)).Target = this.Instance;
					};
			else if (backReferenceMemberInfo.MemberType == typeof (XPWeakReferenceStruct))
				list.ListChanged += (sender, args) =>
					{
						if (args.ListChangedType != ListChangedType.ItemAdded) return;

						var obj = list[args.NewIndex];
						backReferenceMemberInfo.SetValue(obj, XPWeakReferenceStruct.SetTargetObject(this.Instance));
					};
			// This code founded in XPBaseObject.CreateCollection<>() method
			GC.SuppressFinalize(list);

			return list;
		}

		public virtual void OnDeleted()
		{
			foreach (var member in this.TypeInfo.Members.Where(a => a.IsAggregated && a.IsList && a.ListElementTypeInfo != null && a.ListElementTypeInfo.IsDomainComponent))
				this.ObjectSpace.Delete(member.GetValue(this.Instance));
		}
	}
}