﻿using System;
using System.Collections.Generic;
using System.Linq;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.DC;
using DevExpress.Persistent.Base;
using DevExpress.Xpo.Helpers;

namespace Xafari.Base
{

	public static class TypeExtensions
	{

		public static IMemberInfo GetTermMember<TAttribute>(this Type type, Func<TAttribute, bool> comparer = null)
			where TAttribute : TermAttribute
		{
			if (type == null) throw new ArgumentNullException("type");

			var typeInfo = XafTypesInfo.Instance.FindTypeInfo(type);
			if (typeInfo == null) return null;

			return typeInfo.Members
				.FirstOrDefault(a =>
					{
						var attr = a.FindAttribute<TAttribute>();
						return attr != null && (comparer == null || comparer(attr));
					});
		}

		public static IMemberInfo GetTermMember<TAttribute>(this ITypeInfo type, Func<TAttribute, bool> comparer = null)
			where TAttribute : TermAttribute
		{
			return type.Type.GetTermMember(comparer);
		}

		public static IMemberInfo GetTermMember(this Type type, string termName, Func<TermAttribute, bool> comparer = null)
		{
			if (type == null) throw new ArgumentNullException("type");

			var typeInfo = XafTypesInfo.Instance.FindTypeInfo(type);
			if (typeInfo == null) return null;

			return typeInfo.Members
				       .FirstOrDefault(a =>
					       {
						       var attr = a.FindAttribute<TermAttribute>();
						       return attr != null && attr.TermName == termName && (comparer == null || comparer(attr));
					       }) ?? typeInfo.Members.FirstOrDefault(a => a.Name == termName);
		}

		public static TValue GetValueByTerm<TValue>(this object obj, string termName)
		{
			var termMember = obj.GetType().GetTermMember(termName);
			if (termMember == null) return default(TValue);

			return (TValue)termMember.GetValue(obj);
		}

		public static IList<TValue> GetListValueByTerm<TValue>(this object obj, string termName)
		{
			var termMember = obj.GetType().GetTermMember(termName);
			if (termMember == null) return null;

			var listMorpherType = typeof (ListMorpher<,>).MakeGenericType(typeof (TValue), termMember.ListElementType);
			var listMorpher = ReflectionHelper.CreateObject(listMorpherType, termMember.GetValue(obj));

			return (IList<TValue>)listMorpher;
		}

		public static void SetValueByTerm<TValue>(this object obj, string termName, TValue value)
		{
			var termMember = obj.GetType().GetTermMember(termName);
			if (termMember == null) return;

			termMember.SetValue(obj, value);
		}
	}
}