﻿using System;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.Xpo;
using DevExpress.Xpo;
using DevExpress.Xpo.Helpers;

namespace NonPersistentCollections.Module
{
	public struct XPWeakReferenceStruct
	{
		[Persistent]
		public XPObjectType TargetType { get; set; }
		[Persistent]
		public string TargetKey
		{
			get
			{
				return XPWeakReference.KeyToString(this.TargetKeyValue);
			}
			set
			{
				this.TargetKeyValue = XPWeakReference.StringToKey(value);
			}
		}

		[NonPersistent]
		private object TargetKeyValue;

		public object GetTargetObject(IObjectSpace objectSpace)
		{
			var xpObjectSpace = objectSpace as XPObjectSpace;
			return xpObjectSpace == null ? null : this.GetTargetObject(xpObjectSpace.Session);
		}

		public object GetTargetObject(Session session)
		{
			if (this.TargetKeyValue == null)
				return null;

			if (!this.TargetType.IsValidType)
				return null;

			return session == null
				? null
				: session.GetObjectByKey(this.TargetType.TypeClassInfo, this.TargetKeyValue);
		}

		public static XPWeakReferenceStruct SetTargetObject(object value)
		{
			return new XPWeakReferenceStruct {Target = value};
		}

		public object Target
		{
			set
			{
				if (value == null)
				{
					this.TargetType = null;
					this.TargetKeyValue = null;
				}
				else
				{
					var sessionProvider = value as ISessionProvider;

					if (sessionProvider == null)
						throw new InvalidOperationException(string.Format("NonPersistent object '{0}' is not supported",
						                                                  value.GetType().FullName));

					var session = sessionProvider.Session;

					if (session == null)
						throw new InvalidOperationException("Session is null");

					if (session.IsNewObject(value))
						throw new ArgumentException("Can't use nonsaved object");

					this.TargetType = session.GetObjectType(value);
					this.TargetKeyValue = session.GetKeyValue(value);
				}
			}
		}
	}
}